@@ -18,7 +18,7 @@ my $BIN_DIR = "__BIN_DIR";
 my $SCRIPT_DIR = "__SCRIPT_DIR";
 
 my $VERSION_INFO = q~
-DNAdiff version 1.00
+DNAdiff version 1.2
     ~;
 
 my $HELP_INFO = q~
@@ -209,7 +209,8 @@ sub MakeReport()
     my ($rqnAligns1, $rqnAlignsM) = (0,0);  # alignment counter
     my ($rSumLen1, $qSumLen1) = (0,0);      # alignment length sum
     my ($rSumLenM, $qSumLenM) = (0,0);      # alignment length sum
-    my ($rqSumIdy1, $rqSumIdyM) = (0,0);    # alignment identity sum
+    my ($rqSumLen1, $rqSumLenM) = (0,0);    # combined alignment length sum
+    my ($rqSumIdy1, $rqSumIdyM) = (0,0);    # weighted alignment identity sum
     my ($qnIns, $rnIns) = (0,0);            # insertion count
     my ($qSumIns, $rSumIns) = (0,0);        # insertion length sum
     my ($qnTIns, $rnTIns) = (0,0);          # tandem insertion count
@@ -225,17 +226,18 @@ sub MakeReport()
     my ($rqnSNPs, $rqnIndels) = (0,0);      # snp and indel counts
     my ($rqnGSNPs, $rqnGIndels) = (0,0);    # good snp and indel counts
     my %rqSNPs =                            # SNP hash
-        ( "."=>{"A"=>0,"C"=>0,"G"=>0,"T"=>0},
-          "A"=>{"."=>0,"C"=>0,"G"=>0,"T"=>0},
-          "C"=>{"."=>0,"A"=>0,"G"=>0,"T"=>0},
-          "G"=>{"."=>0,"A"=>0,"C"=>0,"T"=>0},
-          "T"=>{"."=>0,"A"=>0,"C"=>0,"G"=>0} );
+      ( "."=>{"A"=>0,"C"=>0,"G"=>0,"T"=>0},
+        "A"=>{"."=>0,"C"=>0,"G"=>0,"T"=>0},
+        "C"=>{"."=>0,"A"=>0,"G"=>0,"T"=>0},
+        "G"=>{"."=>0,"A"=>0,"C"=>0,"T"=>0},
+        "T"=>{"."=>0,"A"=>0,"C"=>0,"G"=>0} );
     my %rqGSNPs =                           # good SNP hash
-        ( "."=>{"A"=>0,"C"=>0,"G"=>0,"T"=>0},
-          "A"=>{"."=>0,"C"=>0,"G"=>0,"T"=>0},
-          "C"=>{"."=>0,"A"=>0,"G"=>0,"T"=>0},
-          "G"=>{"."=>0,"A"=>0,"C"=>0,"T"=>0},
-          "T"=>{"."=>0,"A"=>0,"C"=>0,"G"=>0} );
+      ( "."=>{"A"=>0,"C"=>0,"G"=>0,"T"=>0},
+        "A"=>{"."=>0,"C"=>0,"G"=>0,"T"=>0},
+        "C"=>{"."=>0,"A"=>0,"G"=>0,"T"=>0},
+        "G"=>{"."=>0,"A"=>0,"C"=>0,"T"=>0},
+        "T"=>{"."=>0,"A"=>0,"C"=>0,"G"=>0} );
+
     my $header;                             # delta header
 
     #-- Get delta header
@@ -271,7 +273,8 @@ sub MakeReport()
         $rqnAlignsM++;
         $rSumLenM += $A[4];
         $qSumLenM += $A[5];
-        $rqSumIdyM += $A[6];
+        $rqSumIdyM += ($A[6] / 100.0) * ($A[4] + $A[5]);
+        $rqSumLenM += ($A[4] + $A[5]);
 
         #-- If new ID, add to sequence and base count
         if ( $refs{$A[11]} > 0 ) {
@@ -287,13 +290,13 @@ sub MakeReport()
 
         #-- Add to breakpoint counts
         my ($lo, $hi);
-        if ( $A[1] < $A[2] ) { $lo = $A[1]; $hi = $A[2]; }
-        else                 { $lo = $A[2]; $hi = $A[1]; }
+        if ( $A[0] < $A[1] ) { $lo = $A[0]; $hi = $A[1]; }
+        else                 { $lo = $A[1]; $hi = $A[0]; }
         $rnBrk++ if ( $lo != 1 );
         $rnBrk++ if ( $hi != $A[7] );
 
-        if ( $A[3] < $A[4] ) { $lo = $A[3]; $hi = $A[4]; }
-        else                 { $lo = $A[4]; $hi = $A[3]; }
+        if ( $A[2] < $A[3] ) { $lo = $A[2]; $hi = $A[3]; }
+        else                 { $lo = $A[3]; $hi = $A[2]; }
         $qnBrk++ if ( $lo != 1 );
         $qnBrk++ if ( $hi != $A[8] );
     }
@@ -311,7 +314,8 @@ sub MakeReport()
         $rqnAligns1++;
         $rSumLen1 += $A[4];
         $qSumLen1 += $A[5];
-        $rqSumIdy1 += $A[6];
+        $rqSumIdy1 += ($A[6] / 100.0) * ($A[4] + $A[5]);
+        $rqSumLen1 += ($A[4] + $A[5]);
     }
     FileClose($fhi, $OPT_CoordsFile1);
 
@@ -338,8 +342,10 @@ sub MakeReport()
              }
         }
 
-        #-- "unalign" sequence
-        $rnABases -= $gap if ( $gap > 0 );
+        #-- Remove unaligned sequence from count
+        if ( $A[1] ne "DUP" ) {
+          $rnABases -= $gap if ( $gap > 0 );
+        }
 
         #-- Add to insertion count
         if ( $ins > 0 ) {
@@ -353,7 +359,7 @@ sub MakeReport()
         $rnTrn++ if ( $A[1] eq "SEQ" );
     }
     FileClose($fhi, $OPT_DiffRFile);
-    
+
     #-- Count query diff features and indels
     $fhi = FileOpen("<", $OPT_DiffQFile);
     while (<$fhi>) {
@@ -375,8 +381,10 @@ sub MakeReport()
             }
         }
 
-        #-- "unalign" sequence
-        $qnABases -= $gap if ( $gap > 0 );
+        #-- Remove unaligned sequence from count
+        if ( $A[1] ne "DUP" ) {
+          $qnABases -= $gap if ( $gap > 0 );
+        }
 
         #-- Add to insertion count
         if ( $ins > 0 ) {
@@ -404,12 +412,14 @@ sub MakeReport()
 
         #-- Plain SNPs
         $rqSNPs{$r}{$q}++;
+        if ( !exists($rqSNPs{$q}{$r}) ) { $rqSNPs{$q}{$r} = 0; }
         if ( $r eq '.' || $q eq '.' ) { $rqnIndels++; }
         else                          { $rqnSNPs++; }
 
         #-- Good SNPs with sufficient match buffer
         if ( $A[4] >= $SNPBuff ) {
             $rqGSNPs{$r}{$q}++;
+            if ( !exists($rqGSNPs{$q}{$r}) ) { $rqGSNPs{$q}{$r} = 0; }
             if ( $r eq '.' || $q eq '.' ) { $rqnGIndels++; }
             else                          { $rqnGSNPs++; }
         }
@@ -441,7 +451,7 @@ sub MakeReport()
      ( sprintf "%10d(%.2f%%)",
        $qnSeqs - $qnASeqs,
        ($qnSeqs ? ($qnSeqs - $qnASeqs) / $qnSeqs * 100.0 : 0) );
-     
+
     print  $fho "\n[Bases]\n";
 
     printf $fho "%-15s %20d %20d\n",
@@ -473,8 +483,8 @@ sub MakeReport()
     ($rqnAligns1 ? $qSumLen1 / $rqnAligns1 : 0);
     printf $fho "%-15s %20.2f %20.2f\n",
     "AvgIdentity",
-    ($rqnAligns1 ? $rqSumIdy1 / $rqnAligns1 : 0),
-    ($rqnAligns1 ? $rqSumIdy1 / $rqnAligns1 : 0);
+    ($rqSumLen1 ? $rqSumIdy1 / $rqSumLen1 * 100.0 : 0),
+    ($rqSumLen1 ? $rqSumIdy1 / $rqSumLen1 * 100.0 : 0);
 
     print  $fho "\n";
 
@@ -488,8 +498,8 @@ sub MakeReport()
     ($rqnAlignsM ? $qSumLenM / $rqnAlignsM : 0);
     printf $fho "%-15s %20.2f %20.2f\n",
     "AvgIdentity",
-    ($rqnAlignsM ? $rqSumIdyM / $rqnAlignsM : 0),
-    ($rqnAlignsM ? $rqSumIdyM / $rqnAlignsM : 0);
+    ($rqSumLenM ? $rqSumIdyM / $rqSumLenM * 100.0 : 0),
+    ($rqSumLenM ? $rqSumIdyM / $rqSumLenM * 100.0 : 0);
 
     print  $fho "\n[Feature Estimates]\n";
 
@@ -528,76 +538,104 @@ sub MakeReport()
 
     printf $fho "%-15s %20d %20d\n",
     "TotalSNPs", $rqnSNPs, $rqnSNPs;
-    foreach my $r ("A","C") {
-        foreach my $q ("A","C","G","T") {
-            if ( $r ne $q ) {
-                printf $fho "%-15s %20s %20s\n",
-                "$r$q",
-                ( sprintf "%10d(%.2f%%)",
-                  $rqSNPs{$r}{$q},
-                  ($rqnSNPs ? $rqSNPs{$r}{$q} / $rqnSNPs * 100.0 : 0) ),
-                ( sprintf "%10d(%.2f%%)",
-                  $rqSNPs{$q}{$r},
-                  ($rqnSNPs ? $rqSNPs{$q}{$r} / $rqnSNPs * 100.0 : 0) );
-            }
+    foreach my $r (keys %rqSNPs) {
+      foreach my $q (keys %{$rqSNPs{$r}}) {
+        if ( $r ne "." && $q ne "." ) {
+          printf $fho "%-15s %20s %20s\n",
+            "$r$q",
+              ( sprintf "%10d(%.2f%%)",
+                $rqSNPs{$r}{$q},
+                ($rqnSNPs ? $rqSNPs{$r}{$q} / $rqnSNPs * 100.0 : 0) ),
+                  ( sprintf "%10d(%.2f%%)",
+                    $rqSNPs{$q}{$r},
+                    ($rqnSNPs ? $rqSNPs{$q}{$r} / $rqnSNPs * 100.0 : 0) );
         }
+      }
     }
 
     print  $fho "\n";
 
     printf $fho "%-15s %20d %20d\n",
     "TotalGSNPs", $rqnGSNPs, $rqnGSNPs;
-    foreach my $r ("A","C") {
-        foreach my $q ("A","C","G","T") {
-            if ( $r ne $q ) {
-                printf $fho "%-15s %20s %20s\n",
-                "$r$q",
-                ( sprintf "%10d(%.2f%%)",
-                  $rqGSNPs{$r}{$q},
-                  ($rqnGSNPs ? $rqGSNPs{$r}{$q} / $rqnGSNPs * 100.0 : 0) ),
-                ( sprintf "%10d(%.2f%%)",
-                  $rqGSNPs{$q}{$r},
-                  ($rqnGSNPs ? $rqGSNPs{$q}{$r} / $rqnGSNPs * 100.0 : 0) );
-            }
+    foreach my $r (keys %rqGSNPs) {
+      foreach my $q (keys %{$rqGSNPs{$r}}) {
+        if ( $r ne "." && $q ne "." ) {
+          printf $fho "%-15s %20s %20s\n",
+            "$r$q",
+              ( sprintf "%10d(%.2f%%)",
+                $rqGSNPs{$r}{$q},
+                ($rqnGSNPs ? $rqGSNPs{$r}{$q} / $rqnGSNPs * 100.0 : 0) ),
+                  ( sprintf "%10d(%.2f%%)",
+                    $rqGSNPs{$q}{$r},
+                    ($rqnGSNPs ? $rqGSNPs{$q}{$r} / $rqnGSNPs * 100.0 : 0) );
         }
+      }
     }
 
     print  $fho "\n";
 
     printf $fho "%-15s %20d %20d\n",
     "TotalIndels", $rqnIndels, $rqnIndels;
-    foreach my $r ("A","C","G","T") {
-        foreach my $q (".") {
-            if ( $r ne $q ) {
-                printf $fho "%-15s %20s %20s\n",
-                "$r$q",
-                ( sprintf "%10d(%.2f%%)",
-                  $rqSNPs{$r}{$q},
-                  ($rqnIndels ? $rqSNPs{$r}{$q} / $rqnIndels * 100.0 : 0) ),
-                ( sprintf "%10d(%.2f%%)",
-                  $rqSNPs{$q}{$r},
-                  ($rqnIndels ? $rqSNPs{$q}{$r} / $rqnIndels * 100.0 : 0) );
-            }
+    foreach my $r (keys %rqSNPs) {
+      foreach my $q (keys %{$rqSNPs{$r}}) {
+        if ( $q eq "." ) {
+          printf $fho "%-15s %20s %20s\n",
+            "$r$q",
+              ( sprintf "%10d(%.2f%%)",
+                $rqSNPs{$r}{$q},
+                ($rqnIndels ? $rqSNPs{$r}{$q} / $rqnIndels * 100.0 : 0) ),
+                  ( sprintf "%10d(%.2f%%)",
+                    $rqSNPs{$q}{$r},
+                    ($rqnIndels ? $rqSNPs{$q}{$r} / $rqnIndels * 100.0 : 0) );
         }
+      }
+    }
+    foreach my $r (keys %rqSNPs) {
+      foreach my $q (keys %{$rqSNPs{$r}}) {
+        if ( $r eq "." ) {
+          printf $fho "%-15s %20s %20s\n",
+            "$r$q",
+              ( sprintf "%10d(%.2f%%)",
+                $rqSNPs{$r}{$q},
+                ($rqnIndels ? $rqSNPs{$r}{$q} / $rqnIndels * 100.0 : 0) ),
+                  ( sprintf "%10d(%.2f%%)",
+                    $rqSNPs{$q}{$r},
+                    ($rqnIndels ? $rqSNPs{$q}{$r} / $rqnIndels * 100.0 : 0) );
+        }
+      }
     }
 
     print  $fho "\n";
 
     printf $fho "%-15s %20d %20d\n",
     "TotalGIndels", $rqnGIndels, $rqnGIndels;
-    foreach my $r ("A","C","G","T") {
-        foreach my $q (".") {
-            if ( $r ne $q ) {
-                printf $fho "%-15s %20s %20s\n",
-                "$r$q",
-                ( sprintf "%10d(%.2f%%)",
-                  $rqGSNPs{$r}{$q},
-                  ($rqnGIndels ? $rqGSNPs{$r}{$q} / $rqnGIndels * 100.0 : 0) ),
-                ( sprintf "%10d(%.2f%%)",
-                  $rqGSNPs{$q}{$r},
-                  ($rqnGIndels ? $rqGSNPs{$q}{$r} / $rqnGIndels * 100.0 : 0) );
-            }
+    foreach my $r (keys %rqGSNPs) {
+      foreach my $q (keys %{$rqGSNPs{$r}}) {
+        if ( $q eq "." ) {
+          printf $fho "%-15s %20s %20s\n",
+            "$r$q",
+              ( sprintf "%10d(%.2f%%)",
+                $rqGSNPs{$r}{$q},
+                ($rqnGIndels ? $rqGSNPs{$r}{$q} / $rqnGIndels * 100.0 : 0) ),
+                  ( sprintf "%10d(%.2f%%)",
+                    $rqGSNPs{$q}{$r},
+                    ($rqnGIndels ? $rqGSNPs{$q}{$r} / $rqnGIndels * 100.0 : 0) );
         }
+      }
+    }
+    foreach my $r (keys %rqGSNPs) {
+      foreach my $q (keys %{$rqGSNPs{$r}}) {
+        if ( $r eq "." ) {
+          printf $fho "%-15s %20s %20s\n",
+            "$r$q",
+              ( sprintf "%10d(%.2f%%)",
+                $rqGSNPs{$r}{$q},
+                ($rqnGIndels ? $rqGSNPs{$r}{$q} / $rqnGIndels * 100.0 : 0) ),
+                  ( sprintf "%10d(%.2f%%)",
+                    $rqGSNPs{$q}{$r},
+                    ($rqnGIndels ? $rqGSNPs{$q}{$r} / $rqnGIndels * 100.0 : 0) );
+        }
+      }
     }
 
     FileClose($fho, $OPT_ReportFile);
@@ -607,14 +645,14 @@ sub MakeReport()
     if ( $rnSeqs != $rnASeqs ) {
         $fho = FileOpen(">", $OPT_UnRefFile);
         while ( my ($key, $val) = each(%refs) ) {
-            print $fho "$key\t$val\n" unless $val < 0;
+            print $fho "$key\tUNI\t1\t$val\t$val\n" unless $val < 0;
         }
         FileClose($fho, $OPT_UnRefFile);
     }
     if ( $qnSeqs != $qnASeqs ) {
         $fho = FileOpen(">", $OPT_UnQryFile);
         while ( my ($key, $val) = each(%qrys) ) {
-            print $fho "$key\t$val\n" unless $val < 0;
+            print $fho "$key\tUNI\t1\t$val\t$val\n" unless $val < 0;
         }
         FileClose($fho, $OPT_UnQryFile);
     }
@@ -681,7 +719,7 @@ sub GetOpt()
         print STDERR "ERROR: TIGR::Foundation could not be initialized";
         exit(1);
     }
-    
+
     #-- Set help and usage information
     $TIGR->setHelpInfo($HELP_INFO);
     $TIGR->setUsageInfo($USAGE_INFO);
@@ -694,7 +732,7 @@ sub GetOpt()
          "d|delta=s"  => \$OPT_DeltaFile,
          "p|prefix=s" => \$OPT_Prefix,
          );
-    
+
     #-- Check if the parsing was successful
     if ( $err
          || (defined($OPT_DeltaFile) && scalar(@ARGV) != 0)
