@@ -99,6 +99,8 @@ my $OPT_Pfile;                     # .ps .png output
 
 my $OPT_gpstatus;                  # gnuplot status
 
+my $OPT_ONLY_USE_FATTEST;          # Only use fattest alignment for layout
+
 
 #============================================================== Foundation ====#
 my $VERSION = '3.5';
@@ -146,6 +148,7 @@ my $HELP = qq~
     -l
     --layout        Layout a .delta multiplot in an intelligible fashion,
                     this option requires the -R -Q options
+    --fat           Layout sequences using fattest alignment only
     -p|prefix       Set the prefix of the output files (default '$OPT_prefix')
     -rv             Reverse video for x11 plots
     -r|IdR          Plot a particular reference sequence ID on the X-axis
@@ -289,6 +292,7 @@ MAIN:
         #-- parent runs gnuplot
         if ( $child ) {
             RunGP( );
+            kill 1, $child;
         }
         #-- child listens to clipboard
         elsif ( defined $child ) {
@@ -380,6 +384,9 @@ sub ParseIDs ($$)
     my $isfasta;
     my $offset = 0;
     while ( <IDFILE> ) {
+        #-- Ignore blank lines
+        if ( /^\s*$/ ) { next; }
+
         #-- FastA header
         if ( /^>(\S+)/ ) {
             if ( exists $href->{$1} ) {
@@ -694,6 +701,28 @@ sub LayoutIDs ($$)
         $loQ = $dQ == 1 ? $sQ : $eQ;
         $hiQ = $dQ == 1 ? $eQ : $sQ;
 
+        if ($OPT_ONLY_USE_FATTEST)
+        {
+          #-- Check to see if there is another better alignment
+          if (exists $qc{$idQ})
+          {
+            my ($oldR) = keys %{$qc{$idQ}[2]};
+            my $val = $qc{$idQ}[2]{$oldR};
+
+            if (${$val->[4]} - ${$val->[3]} > $hiR - $loR)
+            {
+              #-- Old alignment is better, skip this one
+              next;
+            }
+            else
+            {
+              #-- This alignment is better, prune old alignment
+              delete $rc{$oldR}[2]{$idQ};
+              delete $qc{$idQ};
+            }
+          }
+        }
+
         #-- initialize
         if ( !exists $rc{$idR} ) { $rc{$idR} = [ 0, $lenR, { } ]; }
         if ( !exists $qc{$idQ} ) { $qc{$idQ} = [ 0, $lenQ, { } ]; }
@@ -1383,12 +1412,12 @@ sub ListenGP($$)
             print "$refid\t$qryid\t$refc\t$qryc\n";
 
             if ( $rsock ) {
-                print $rsock "contig E$refid $refc\n";
-                print "sent \"contig E$refid $refc\" to $OPT_rport\n";
+                print $rsock "contig I$refid $refc\n";
+                print "sent \"contig I$refid $refc\" to $OPT_rport\n";
             }
             if ( $qsock ) {
-                print $qsock "contig E$qryid $qryc\n";
-                print "sent \"contig E$qryid $qryc\" to $OPT_qport\n";
+                print $qsock "contig I$qryid $qryc\n";
+                print "sent \"contig I$qryid $qryc\" to $OPT_qport\n";
             }
         }
 
@@ -1435,6 +1464,7 @@ sub ParseOptions ( )
          "small"        => \$opt_small,
          "medium"       => \$opt_medium,
          "large"        => \$opt_large,
+         "fat"          => \$OPT_ONLY_USE_FATTEST,
          );
 
     if ( !$err  ||  scalar (@ARGV) != 1 ) {
@@ -1524,6 +1554,11 @@ sub ParseOptions ( )
             or die "ERROR: Could not write $OPT_Hfile, $!\n";
     }
 
+    if ($OPT_ONLY_USE_FATTEST)
+    {
+      $OPT_layout = 1;
+    }
+
     if ( $OPT_filter || $OPT_layout ) {
         $OPT_Dfile = $OPT_prefix . $SUFFIX{$FILTER};
         $tigr->isWritableFile($OPT_Dfile) or $tigr->isCreatableFile($OPT_Dfile)
